#*************************************************************************************************************************************
#** Notification before use																											**
#*************************************************************************************************************************************
#** Depending on the type of product you are using, the definitions of Parameter, IO Logic, AxisStatus, etc. may be different.		**
#** This example is based on Ezi-SERVO, so please apply the appropriate value depending on the product you are using.				**
#*************************************************************************************************************************************
#** ex)	FM_EZISERVO_PARAM			// Parameter enum when using Ezi-SERVO						 									**
#**		FM_EZIMOTIONLINK_PARAM		// Parameter enum when using Ezi-MOTIONLINK														**
#*************************************************************************************************************************************

import sys
import os

try:
    library_path = os.path.abspath(
        os.path.join(os.path.dirname(__file__), "..", "..", "Library")
    )
except NameError:
    library_path = os.path.abspath(
        os.path.join(os.getcwd(), "..", "..", "Library")
    )
sys.path.append(library_path)


from FAS_EziMOTIONPlusR import *
from MOTION_DEFINE import *
from ReturnCodes_Define import *
from MOTION_EziSERVO_DEFINE import *

import time

CW = 1
CCW = 0


def Connect(sPort: int, dwBaudRate: int, nPortID: int) -> bool:
    bSuccess = False

    if FAS_Connect(sPort, dwBaudRate, nPortID) == False:
        print("Connection Fail!")
        bSuccess = False

    else:
        print("Conncetion Success!")
        bSuccess = True
    return bSuccess


def CheckDriveErr(nPortID: int, iSlaveNo: int)-> bool:
    # Check Drive's Error
    status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)

    if status_result != FMM_OK:
        print("Function(FAS_GetAxisStatus) was failed.")
        return False

    if axis_status & EZISERVO_AXISSTATUS.FFLAG_ERRORALL:
        # if Drive's Error was detected, Reset the ServoAlarm
        if FAS_ServoAlarmReset(nPortID, iSlaveNo) != FMM_OK:
            print("Function(FAS_ServoAlarmReset) was failed.")
            return False

    return True

def SetServoOn(nPortID: int, iSlaveNo: int)-> bool:
    # Check Drive's Servo Status

    # if ServoOnFlagBit is OFF('0'), switch to ON('1')
    status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)
    if status_result != FMM_OK:
        print("Function(FAS_GetAxisStatus) was failed.")
        return False

    if (axis_status & EZISERVO_AXISSTATUS.FFLAG_SERVOON) == 0:
        if FAS_ServoEnable(nPortID, iSlaveNo, 1) != FMM_OK:
            print("Function(FAS_ServoEnable) was failed.")
            return False

        while (
            axis_status & EZISERVO_AXISSTATUS.FFLAG_SERVOON
        ) == 0:  # Wait until FFLAG_SERVOON is ON
            time.sleep(0.001)

            status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)
            if status_result != FMM_OK:
                print("Function(FAS_GetAxisStatus) was failed.")
                return False

            if (axis_status & EZISERVO_AXISSTATUS.FFLAG_SERVOON) != 0:
                print("Servo ON")

    else:
        print("Servo is already ON")

    return True

def SetOriginParameter(nPortID: int, iSlaveNo: int)-> bool:
    
    # Set Origin Parameter
    nOrgSpeed = 50000
    nOrgSearchSpeed = 1000
    nOrgAccDecTime = 50
    nOrgMethod = (
        2  # Origin Method = 2 is 'Limit Origin' in the Ezi-SERVOII Plus-E model
    )
    nOrgDir = CW
    nOrgOffset = 0
    nOrgPositionSet = 0
    nOrgTorqueRatio = 50

    if FAS_SetParameter(nPortID, iSlaveNo, SERVO_ORGSPEED, nOrgSpeed) != FMM_OK:
        print("Function(FAS_SetParameter[SERVO_ORGSPEED]) was failed.")
        return False

    if FAS_SetParameter(nPortID, iSlaveNo, SERVO_ORGSEARCHSPEED, nOrgSearchSpeed) != FMM_OK:
        print("Function(FAS_SetParameter[SERVO_ORGSEARCHSPEED]) was failed.")
        return False

    if FAS_SetParameter(nPortID, iSlaveNo, SERVO_ORGACCDECTIME, nOrgAccDecTime) != FMM_OK:
        print("Function(FAS_SetParameter[SERVO_ORGACCDECTIME]) was failed.")
        return False

    if FAS_SetParameter(nPortID, iSlaveNo, SERVO_ORGMETHOD, nOrgMethod) != FMM_OK:
        print("Function(FAS_SetParameter[SERVO_ORGMETHOD]) was failed.")
        return False

    if FAS_SetParameter(nPortID, iSlaveNo, SERVO_ORGDIR, nOrgDir) != FMM_OK:
        print("Function(FAS_SetParameter[SERVO_ORGDIR]) was failed.")
        return False

    if FAS_SetParameter(nPortID, iSlaveNo, SERVO_ORGOFFSET, nOrgOffset) != FMM_OK:
        print("Function(FAS_SetParameter[SERVO_ORGOFFSET]) was failed.")
        return False

    if FAS_SetParameter(nPortID, iSlaveNo, SERVO_ORGPOSITIONSET, nOrgPositionSet) != FMM_OK:
        print("Function(FAS_SetParameter[SERVO_ORGPOSITIONSET]) was failed.")
        return False

    if FAS_SetParameter(nPortID, iSlaveNo, SERVO_ORGTORQUERATIO, nOrgTorqueRatio) != FMM_OK:
        print("Function(FAS_SetParameter[SERVO_ORGTORQUERATIO]) was failed.")
        return False

    return True


def OriginSearch(nPortID: int, iSlaveNo: int) -> bool:

    # Act Origin Search Function
    print("---------------------------")

    # Origin Search Start
    if FAS_MoveOriginSingleAxis(nPortID, iSlaveNo) != FMM_OK:
        print("Function(FAS_MoveOriginSingleAxis) was failed.")
        return False

    # Check the Axis status until OriginReturning value is released.

    while True:
        time.sleep(0.001)

        status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)
        if status_result != FMM_OK:
            print("Function(FAS_GetAxisStatus) was failed.")
            return False

        if not (axis_status & EZISERVO_AXISSTATUS.FFLAG_ORIGINRETURNING):
            break

    if axis_status & EZISERVO_AXISSTATUS.FFLAG_ORIGINRETOK:
        print("Origin Search Success!")
        return True
    else:
        print("Origin Search Fail!")
        return False


def main():
    sPort = "ttyS0"
    iSlaveNo = 0
    dwBaudRate = 115200
    nPortID = 0

    # Device Connect
    if not Connect(sPort, dwBaudRate, nPortID):
        input("Press Enter to exit...")
        exit(1)

    # Drive Error Check
    if not CheckDriveErr(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # ServoOn
    if not SetServoOn(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Set Origin Parameter
    if not SetOriginParameter(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Act Origin Search Function
    if not OriginSearch(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Connection Close
    FAS_Close(nPortID)

    input("Press Enter to exit...")


if __name__ == "__main__":
    main()
