#*************************************************************************************************************************************
#** Notification before use																											**
#*************************************************************************************************************************************
#** Depending on the type of product you are using, the definitions of Parameter, IO Logic, AxisStatus, etc. may be different.		**
#** This example is based on Ezi-SERVO, so please apply the appropriate value depending on the product you are using.				**
#*************************************************************************************************************************************
#** ex)	FM_EZISERVO_PARAM			// Parameter enum when using Ezi-SERVO						 									**
#**		FM_EZIMOTIONLINK_PARAM		// Parameter enum when using Ezi-MOTIONLINK														**
#*************************************************************************************************************************************

import sys
import os

try:
    library_path = os.path.abspath(
        os.path.join(os.path.dirname(__file__), "..", "..", "Library")
    )
except NameError:
    library_path = os.path.abspath(
        os.path.join(os.getcwd(), "..", "..", "Library")
    )
sys.path.append(library_path)

from FAS_EziMOTIONPlusR import *
from MOTION_DEFINE import *
from ReturnCodes_Define import *
from MOTION_EziSERVO_DEFINE import *

import time


def Connect(sPort: int, dwBaudRate: int, nPortID: int) -> bool:
    bSuccess = False

    if FAS_Connect(sPort, dwBaudRate, nPortID) == False:
        print("Connection Fail!")
        bSuccess = False

    else:
        print("Conncetion Success!")
        bSuccess = True
    return bSuccess


def CheckDriveErr(nPortID: int, iSlaveNo: int)-> bool:
    # Check Drive's Error
    status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)

    if status_result != FMM_OK:
        print("Function(FAS_GetAxisStatus) was failed.")
        return False

    if axis_status & EZISERVO_AXISSTATUS.FFLAG_ERRORALL:
        # if Drive's Error was detected, Reset the ServoAlarm
        if FAS_ServoAlarmReset(nPortID, iSlaveNo) != FMM_OK:
            print("Function(FAS_ServoAlarmReset) was failed.")
            return False

    return True

def SetServoOn(nPortID: int, iSlaveNo: int)-> bool:
    # Check Drive's Servo Status

    # if ServoOnFlagBit is OFF('0'), switch to ON('1')
    status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)
    if status_result != FMM_OK:
        print("Function(FAS_GetAxisStatus) was failed.")
        return False

    if (axis_status & EZISERVO_AXISSTATUS.FFLAG_SERVOON) == 0:
        if FAS_ServoEnable(nPortID, iSlaveNo, 1) != FMM_OK:
            print("Function(FAS_ServoEnable) was failed.")
            return False

        while (
            axis_status & EZISERVO_AXISSTATUS.FFLAG_SERVOON
        ) == 0:  # Wait until FFLAG_SERVOON is ON
            time.sleep(0.001)

            status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)
            if status_result != FMM_OK:
                print("Function(FAS_GetAxisStatus) was failed.")
                return False

            if (axis_status & EZISERVO_AXISSTATUS.FFLAG_SERVOON) != 0:
                print("Servo ON")

    else:
        print("Servo is already ON")

    return True

def SetPosTable(nPortID: int, iSlaveNo: int)-> bool:

    # Sets values in the position table.
    nodeItem = ITEM_NODE()
    wItemNo = 1

    # Gets the data values of that node.
    status_result, nodeItem = FAS_PosTableReadItem(nPortID, iSlaveNo, wItemNo)
    if status_result != FMM_OK:
        print("Function(FAS_PosTableReadItem) was failed.")
        return False

    # Overwrite part of node's data
    nodeItem.dwMoveSpd = 50000
    nodeItem.lPosition = 250000
    nodeItem.wBranch = 3
    nodeItem.wContinuous = 1

    # Set node data in Position Table
    if FAS_PosTableWriteItem(nPortID, iSlaveNo, wItemNo, nodeItem):
        print("Function(FAS_PosTableWriteItem) was failed.")
        return False
    else:
        print("Setting PosTable Data!")

    return True

def RunPosTable(nPortID: int, iSlaveNo: int)-> bool:

    wItemNo = 1

    print("---------------------------")

    if FAS_PosTableRunItem(nPortID, iSlaveNo, wItemNo) != FMM_OK:
        print("Function(FAS_PosTableRunItem) was failed.")
        return False

    print("PosTable Run!")

    while True:
        time.sleep(0.001)

        status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)
        if status_result != FMM_OK:
            print("Function(FAS_GetAxisStatus) was failed.")
            return False

        if axis_status & EZISERVO_AXISSTATUS.FFLAG_PTSTOPPED:
            print("Position Table Run Stop!")
            break
    return True

def main():
    sPort = "ttyS0"
    iSlaveNo = 0
    dwBaudRate = 115200
    nPortID = 0

    # Device Connect
    if not Connect(sPort, dwBaudRate, nPortID):
        input("Press Enter to exit...")
        exit(1)

    # Drive Error Check
    if not CheckDriveErr(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # ServoOn
    if not SetServoOn(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Sets values in the position table.
    if not SetPosTable(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Run Position Table
    if not RunPosTable(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Connection Close
    FAS_Close(nPortID)

    input("Press Enter to exit...")


if __name__ == "__main__":
    main()
