#*************************************************************************************************************************************
#** Notification before use																											**
#*************************************************************************************************************************************
#** Depending on the type of product you are using, the definitions of Parameter, IO Logic, AxisStatus, etc. may be different.		**
#** This example is based on Ezi-SERVO, so please apply the appropriate value depending on the product you are using.				**
#*************************************************************************************************************************************
#** ex)	FM_EZISERVO_PARAM			// Parameter enum when using Ezi-SERVO						 									**
#**		FM_EZIMOTIONLINK_PARAM		// Parameter enum when using Ezi-MOTIONLINK														**
#*************************************************************************************************************************************

import sys
import os

try:
    library_path = os.path.abspath(
        os.path.join(os.path.dirname(__file__), "..", "..", "Library")
    )
except NameError:
    library_path = os.path.abspath(
        os.path.join(os.getcwd(), "..", "..", "Library")
    )
sys.path.append(library_path)

from FAS_EziMOTIONPlusR import *
from MOTION_DEFINE import *
from ReturnCodes_Define import *
from MOTION_EziSERVO_DEFINE import *

import time


def Connect(sPort: int, dwBaudRate: int, nPortID: int) -> bool:
    bSuccess = False

    if FAS_Connect(sPort, dwBaudRate, nPortID) == False:
        print("Connection Fail!")
        bSuccess = False

    else:
        print("Conncetion Success!")
        bSuccess = True
    return bSuccess


def CheckDriveErr(nPortID: int, iSlaveNo: int)-> bool:
    # Check Drive's Error
    status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)

    if status_result != FMM_OK:
        print("Function(FAS_GetAxisStatus) was failed.")
        return False

    if axis_status & EZISERVO_AXISSTATUS.FFLAG_ERRORALL:
        # if Drive's Error was detected, Reset the ServoAlarm
        if FAS_ServoAlarmReset(nPortID, iSlaveNo) != FMM_OK:
            print("Function(FAS_ServoAlarmReset) was failed.")
            return False

    return True

def SetServoOn(nPortID: int, iSlaveNo: int)-> bool:
    # Check Drive's Servo Status

    # if ServoOnFlagBit is OFF('0'), switch to ON('1')
    status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)
    if status_result != FMM_OK:
        print("Function(FAS_GetAxisStatus) was failed.")
        return False

    if (axis_status & EZISERVO_AXISSTATUS.FFLAG_SERVOON) == 0:
        if FAS_ServoEnable(nPortID, iSlaveNo, 1) != FMM_OK:
            print("Function(FAS_ServoEnable) was failed.")
            return False

        while (
            axis_status & EZISERVO_AXISSTATUS.FFLAG_SERVOON
        ) == 0:  # Wait until FFLAG_SERVOON is ON
            time.sleep(0.001)

            status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)
            if status_result != FMM_OK:
                print("Function(FAS_GetAxisStatus) was failed.")
                return False

            if (axis_status & EZISERVO_AXISSTATUS.FFLAG_SERVOON) != 0:
                print("Servo ON")

    else:
        print("Servo is already ON")

    return True

def OperatePushMode(nPortID: int, iSlaveNo: int)-> bool:

     # normal position motion
    dwStartSpd = 1
    dwMoveSpd = 50000
    wAccel = 500
    wDecel = 500
    lPosition = 500000

    # push motion
    dwPushSpd = 2000
    wPushRate = 50
    wPushMode = 0  # Stop Mode Push
    lEndPosition = lPosition + 10000

    print("---------------------------")

    if (
        FAS_MovePush(
            nPortID, 
            iSlaveNo,
            dwStartSpd,
            dwMoveSpd,
            lPosition,
            wAccel,
            wDecel,
            wPushRate,
            dwPushSpd,
            lEndPosition,
            wPushMode,
        )
        != FMM_OK
    ):
        print("Function(FAS_MovePush) was failed.")
        return False

    # Check the Axis status until motor stops and the FFLAG_MOTIONING value is released.

    while True:
        time.sleep(0.001)
        status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)

        if status_result != FMM_OK:
            print("Function(FAS_GetAxisStatus) was failed.")
            return False

        if not (axis_status & EZISERVO_AXISSTATUS.FFLAG_MOTIONING):
            break
    return True

def CheckPushResult(nPortID: int, iSlaveNo: int) -> bool:
    dwOutPut = 0

    print("---------------------------")
    print("Check Result Function")

    status_result, dwOutput = FAS_GetIOOutput(nPortID, iSlaveNo)

    if status_result != FMM_OK:
        print("Function(FAS_GetIOOutput) was failed.")
        return False
    else:
        if dwOutput & SERVO_OUT_LOGIC_RESERVED0:
            print("Work Detected!")
        else:
            print("Work Not Detected!")

    return True


def ReturnPosition(nPortID: int, iSlaveNo: int) -> bool:
    # Return Poisition

    # Move the motor by 0 pulse (target position : Absolute position)
    lAbsPos = 0
    lVelocity = 50000

    time.sleep(0.1)
    print("---------------------------")
    print("Return Position Start!")
    if FAS_MoveSingleAxisAbsPos(nPortID, iSlaveNo, lAbsPos, lVelocity) != FMM_OK:
        print("Function(FAS_MoveSingleAxisAbsPos) was failed.")
        return False

    # Check the Axis status until motor stops and the Inposition value is checked

    while True:
        time.sleep(0.001)

        status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)
        if status_result != FMM_OK:
            print("Funtion(FAS_GetAxisStatus) was failed.")
            return False
        if not (axis_status & EZISERVO_AXISSTATUS.FFLAG_MOTIONING) and (
            axis_status & EZISERVO_AXISSTATUS.FFLAG_INPOSITION
        ):
            break

    return True


def main():
    sPort = "ttyS0"
    iSlaveNo = 0
    dwBaudRate = 115200
    nPortID = 0

    # Device Connect
    if not Connect(sPort, dwBaudRate, nPortID):
        input("Press Enter to exit...")
        exit(1)

    # Drive Error Check
    if not CheckDriveErr(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # ServoOn
    if not SetServoOn(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Operate PushMode
    if not OperatePushMode(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Check PushMotion Result
    if not CheckPushResult(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Return Position
    if not ReturnPosition(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Connection Close
    FAS_Close(nPortID)

    input("Press Enter to exit...")


if __name__ == "__main__":
    main()
