#*************************************************************************************************************************************
#** Notification before use																											**
#*************************************************************************************************************************************
#** Depending on the type of product you are using, the definitions of Parameter, IO Logic, AxisStatus, etc. may be different.		**
#** This example is based on Ezi-SERVO, so please apply the appropriate value depending on the product you are using.				**
#*************************************************************************************************************************************
#** ex)	FM_EZISERVO_PARAM			// Parameter enum when using Ezi-SERVO						 									**
#**		FM_EZIMOTIONLINK_PARAM		// Parameter enum when using Ezi-MOTIONLINK														**
#*************************************************************************************************************************************

import sys
import os

try:
    library_path = os.path.abspath(
        os.path.join(os.path.dirname(__file__), "..", "..", "Library")
    )
except NameError:
    library_path = os.path.abspath(
        os.path.join(os.getcwd(), "..", "..", "Library")
    )
sys.path.append(library_path)

from FAS_EziMOTIONPlusR import *
from MOTION_DEFINE import *
from ReturnCodes_Define import *
from MOTION_EziSERVO_DEFINE import *

import time

SLAVE_CNT = 2

def Connect(sPort: int, dwBaudRate: int, nPortID: int) -> bool:
bSuccess = False

if FAS_Connect(sPort, dwBaudRate, nPortID) == False:
    print("Connection Fail!")
    bSuccess = False

else:
    print("Conncetion Success!")
    bSuccess = True
return bSuccess


def CheckDriveErr(nPortID: int, iSlaveNo: int) -> bool:
    # Check Drive's Error
    status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)

    if status_result != FMM_OK:
        print(f"[Slave : {iSlaveNo} ] Function(FAS_GetAxisStatus) was failed.")
        return False

    if axis_status & EZISERVO_AXISSTATUS.FFLAG_ERRORALL:
        # if Drive's Error was detected, Reset the ServoAlarm
        if FAS_ServoAlarmReset(nPortID, iSlaveNo) != FMM_OK:
            print(f"[Slave : {iSlaveNo} ] Function(FAS_ServoAlarmReset) was failed.")
            return False

    return True


def SetServoOn(nPortID: int, iSlaveNo: int) -> bool:
    # Check Drive's Servo Status
    status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)

    if status_result != FMM_OK:
        print(f"[Slave : {iSlaveNo} ] Function(FAS_GetAxisStatus) was failed.")
        return False

    # if ServoOnFlagBit is OFF('0'), switch to ON('1')
    if (axis_status & EZISERVO_AXISSTATUS.FFLAG_SERVOON) == 0:
        if FAS_ServoEnable(nPortID, iSlaveNo, 1) != FMM_OK:
            print(f"[Slave : {iSlaveNo} ] Function(FAS_ServoEnable) was failed.")
            return False

        while (
            axis_status & EZISERVO_AXISSTATUS.FFLAG_SERVOON
        ) == 0:  # Wait until FFLAG_SERVOON is ON
            time.sleep(0.001)

            status_result, axis_status = FAS_GetAxisStatus(nPortID, iSlaveNo)
            if status_result != FMM_OK:
                print(f"[Slave : {iSlaveNo} ] Function(FAS_GetAxisStatus) was failed.")
                return False

            if (axis_status & EZISERVO_AXISSTATUS.FFLAG_SERVOON) != 0:
                print(f"[Slave : {iSlaveNo} ] Servo ON")

    else:
        print(f"[Slave : {iSlaveNo} ] Servo is already ON")

    return True


def MoveLinearIncPos(nPortID: int, iSlaveNo: int) -> bool:
    # Move Linear IncPos
    lIncPos = [370000, 370000]
    wAccelTime = 100

    print("---------------------------")
    # Increase the motor by 370000 pulse (target position : Relative position)

    lVelocity = 40000

    print("[Linear Inc Mode] Move Motor!")

    if FAS_MoveLinearIncPos2(nPortID, SLAVE_CNT, iSlaveNo, lVelocity, wAccelTime) != FMM_OK:
        print("Function(FAS_MoveLinearIncPos2) was failed.")
        return False
    # Check the Axis status until motor stops and the Inposition value is checked
    for nID in range(SLAVE_CNT):
        while True:
            time.sleep(0.001)
            status_result, axis_status = FAS_GetAxisStatus(nPortID,nID)

            if status_result != FMM_OK:
                print(f"[Slave : {nID} ] Function(FAS_GetAxisStatus) was failed.")
                return False

            if (
                not axis_status & EZISERVO_AXISSTATUS.FFLAG_MOTIONING
                and axis_status & EZISERVO_AXISSTATUS.FFLAG_INPOSITION
            ):
                break
    return True

def MoveLinearAbsPos(nPortID: int, iSlaveNo: int) -> bool:

    # Move Linear AbsPos
    lAbsPos = [0, 0]
    wAccelTime = 100

    print("---------------------------")

    # Move the motor by 0 pulse (target position : Absolute position)
    lVelocity = 40000

    print("[Linear Abs Mode] Move Motor!")
    if FAS_MoveLinearAbsPos2(nPortID, SLAVE_CNT, iSlaveNo, lAbsPos, lVelocity, wAccelTime) != FMM_OK:
        print("Function(FAS_MoveLinearAbsPos2) was failed.")
        return False

    # Check the Axis status until motor stops and the Inposition value is checked
    for nID in range(SLAVE_CNT):
        while True:
            time.sleep(0.001)
            status_result, axis_status = FAS_GetAxisStatus(nPortID,nID)
            if status_result != FMM_OK:
                print(f"[Slave : {nID} ] Function(FAS_GetAxisStatus) was failed.")
                return False

            if (
                not axis_status & EZISERVO_AXISSTATUS.FFLAG_MOTIONING
                and axis_status & EZISERVO_AXISSTATUS.FFLAG_INPOSITION
            ):
                break
    return True


def main():
    sPort = "ttyS0"
    iSlaveNo = [0, 1]
    dwBaudRate = 115200
    nPortID = 0
    
    # Device Connect
    if not Connect(sPort, dwBaudRate, nPortID):
        input("Press Enter to exit...")
        exit(1)
    for nID in range(SLAVE_CNT):
        # Drive Error Check
        if not CheckDriveErr(nPortID, iSlaveNo[nID]):
            input("Press Enter to exit...")
            sys.exit(1)

        # Servo On
        if not SetServoOn(nPortID, iSlaveNo[nID]):
            input("Press Enter to exit...")
            sys.exit(1)

    # Move Linear IncPos
    if not MoveLinearIncPos(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        sys.exit(1)

    # Move Linear AbsPos
    if not MoveLinearAbsPos(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        sys.exit(1)

    # Connection Close
    FAS_Close(nPortID)

    input("Press Enter to exit...")


if __name__ == "__main__":
    main()
